package com.myeclipseide.jpaspring;

import java.util.List;
import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import javax.persistence.Query;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.context.ApplicationContext;
import org.springframework.stereotype.Repository;

/**
 * A data access object (DAO) providing persistence and search support for
 * Productline entities. Transaction control of the save(), update() and
 * delete() operations can directly support Spring container-managed
 * transactions or they can be augmented to handle user-managed Spring
 * transactions. Each of these methods provides additional information for how
 * to configure it for the desired type of transaction control.
 * 
 * @see com.myeclipseide.jpaspring.Productline
 * @author MyEclipse Persistence Tools
 */
@Repository
public class ProductlineDAO implements IProductlineDAO {
	private static final Log logger = LogFactory.getLog(ProductlineDAO.class);
	// property constants
	public static final String TEXTDESCRIPTION = "textdescription";
	public static final String HTMLDESCRIPTION = "htmldescription";

	private EntityManager entityManager;

	@PersistenceContext
	public void setEntityManager(EntityManager entityManager) {
		this.entityManager = entityManager;
	}

	private EntityManager getEntityManager() {
		return entityManager;
	}

	/**
	 * Perform an initial save of a previously unsaved Productline entity. All
	 * subsequent persist actions of this entity should use the #update()
	 * method. This operation must be performed within the a database
	 * transaction context for the entity's data to be permanently saved to the
	 * persistence store, i.e., database. This method uses the
	 * {@link javax.persistence.EntityManager#persist(Object)
	 * EntityManager#persist} operation.
	 * <p>
	 * User-managed Spring transaction example:
	 * 
	 * <pre>
	 * TransactionStatus txn = txManager
	 * 		.getTransaction(new DefaultTransactionDefinition());
	 * ProductlineDAO.save(entity);
	 * txManager.commit(txn);
	 * </pre>
	 * 
	 * @see <a href =
	 *      "http://www.myeclipseide.com/documentation/quickstarts/jpaspring#containermanaged">Spring
	 *      container-managed transaction examples</a>
	 * @param entity
	 *            Productline entity to persist
	 * @throws RuntimeException
	 *             when the operation fails
	 */
	public void save(Productline entity) {
		logger.info("saving Productline instance");
		try {
			getEntityManager().persist(entity);
			logger.info("save successful");
		} catch (RuntimeException re) {
			logger.error("save failed", re);
			throw re;
		}
	}

	/**
	 * Delete a persistent Productline entity. This operation must be performed
	 * within the a database transaction context for the entity's data to be
	 * permanently deleted from the persistence store, i.e., database. This
	 * method uses the {@link javax.persistence.EntityManager#remove(Object)
	 * EntityManager#delete} operation.
	 * <p>
	 * User-managed Spring transaction example:
	 * 
	 * <pre>
	 * TransactionStatus txn = txManager
	 * 		.getTransaction(new DefaultTransactionDefinition());
	 * ProductlineDAO.delete(entity);
	 * txManager.commit(txn);
	 * entity = null;
	 * </pre>
	 * 
	 * @see <a href =
	 *      "http://www.myeclipseide.com/documentation/quickstarts/jpaspring#containermanaged">Spring
	 *      container-managed transaction examples</a>
	 * @param entity
	 *            Productline entity to delete
	 * @throws RuntimeException
	 *             when the operation fails
	 */
	public void delete(Productline entity) {
		logger.info("deleting Productline instance");
		try {
			entity = getEntityManager().getReference(Productline.class,
					entity.getProductline());
			getEntityManager().remove(entity);
			logger.info("delete successful");
		} catch (RuntimeException re) {
			logger.error("delete failed", re);
			throw re;
		}
	}

	/**
	 * Persist a previously saved Productline entity and return it or a copy of
	 * it to the sender. A copy of the Productline entity parameter is returned
	 * when the JPA persistence mechanism has not previously been tracking the
	 * updated entity. This operation must be performed within the a database
	 * transaction context for the entity's data to be permanently saved to the
	 * persistence store, i.e., database. This method uses the
	 * {@link javax.persistence.EntityManager#merge(Object) EntityManager#merge}
	 * operation.
	 * <p>
	 * User-managed Spring transaction example:
	 * 
	 * <pre>
	 * TransactionStatus txn = txManager
	 * 		.getTransaction(new DefaultTransactionDefinition());
	 * entity = ProductlineDAO.update(entity);
	 * txManager.commit(txn);
	 * </pre>
	 * 
	 * @see <a href =
	 *      "http://www.myeclipseide.com/documentation/quickstarts/jpaspring#containermanaged">Spring
	 *      container-managed transaction examples</a>
	 * @param entity
	 *            Productline entity to update
	 * @return Productline the persisted Productline entity instance, may not be
	 *         the same
	 * @throws RuntimeException
	 *             if the operation fails
	 */
	public Productline update(Productline entity) {
		logger.info("updating Productline instance");
		try {
			Productline result = getEntityManager().merge(entity);
			logger.info("update successful");
			return result;
		} catch (RuntimeException re) {
			logger.error("update failed", re);
			throw re;
		}
	}

	public Productline findById(String id) {
		logger.info("finding Productline instance with id: " + id);
		try {
			Productline instance = getEntityManager().find(Productline.class,
					id);
			return instance;
		} catch (RuntimeException re) {
			logger.error("find failed", re);
			throw re;
		}
	}

	/**
	 * Find all Productline entities with a specific property value.
	 * 
	 * @param propertyName
	 *            the name of the Productline property to query
	 * @param value
	 *            the property value to match
	 * @return List<Productline> found by query
	 */
	@SuppressWarnings("unchecked")
	public List<Productline> findByProperty(String propertyName,
			final Object value) {
		logger.info("finding Productline instance with property: "
				+ propertyName + ", value: " + value);
		try {
			final String queryString = "select model from Productline model where model."
					+ propertyName + "= :propertyValue";
			Query query = getEntityManager().createQuery(queryString);
			query.setParameter("propertyValue", value);
			return query.getResultList();
		} catch (RuntimeException re) {
			logger.error("find by property name failed", re);
			throw re;
		}
	}

	public List<Productline> findByTextdescription(Object textdescription) {
		return findByProperty(TEXTDESCRIPTION, textdescription);
	}

	public List<Productline> findByHtmldescription(Object htmldescription) {
		return findByProperty(HTMLDESCRIPTION, htmldescription);
	}

	/**
	 * Find all Productline entities.
	 * 
	 * @return List<Productline> all Productline entities
	 */
	@SuppressWarnings("unchecked")
	public List<Productline> findAll() {
		logger.info("finding all Productline instances");
		try {
			final String queryString = "select model from Productline model";
			Query query = getEntityManager().createQuery(queryString);
			return query.getResultList();
		} catch (RuntimeException re) {
			logger.error("find all failed", re);
			throw re;
		}
	}

	public static IProductlineDAO getFromApplicationContext(
			ApplicationContext ctx) {
		return (IProductlineDAO) ctx.getBean("ProductlineDAO");
	}
}