package com.myeclipseide.jpaspring;

import org.springframework.context.support.ClassPathXmlApplicationContext;
import org.springframework.orm.jpa.JpaTransactionManager;
import org.springframework.transaction.TransactionStatus;
import org.springframework.transaction.support.DefaultTransactionDefinition;

public class RunJPA {
	private static JpaTransactionManager txManager = null;
	private static IProductlineDAO dao = null;

	public static void main(String[] args) {
		/* 1. Initialize the transactionManager and DAO */
		ClassPathXmlApplicationContext ctx = new ClassPathXmlApplicationContext(
				"/applicationContext.xml");
		txManager = ((JpaTransactionManager) ctx.getBean("transactionManager"));
		dao = ProductlineDAO.getFromApplicationContext(ctx);

		/* 2. Create a reference to our ID */
		String productlineID = "Men Shoes";

		/* 3. Save a new productline to the DB */
		saveProductline(productlineID);

		/* 4. Load the productline from DB to make sure it worked */
		loadProductline(productlineID);

		/* 5. Update the productline in the DB and check it */
		updateProductline(productlineID);

		/* 6. Delete the productline from the DB */
		//deleteProductline(productlineID);
		//use container-managed transaction
		JPAServiceBean bean = (JPAServiceBean) ctx.getBean("JPAServiceBean"); 
		bean.deleteProductLine(productlineID);
	}

	private static void saveProductline(String productlineID) {
		/* 1. Create a new Productline instance */
		Productline newProductline = new Productline(productlineID,
				"Shoes for men.", "<strong>Men Shoes</strong>", null);

		/* 2. Store our new product line in the DB */
		TransactionStatus status = txManager
				.getTransaction(new DefaultTransactionDefinition());
		dao.save(newProductline);
		txManager.commit(status);
	}

	private static void loadProductline(String productlineID) {
		/* 1. Now retrieve the new product line, using the ID we created */
		Productline loadedProductline = dao.findById(productlineID);

		/* 2. Print out the product line information */
		System.out.println("*NEW* Product Line [productLine="
				+ loadedProductline.getProductline() + ", textDescription="
				+ loadedProductline.getTextdescription() + "]");
	}

	private static void updateProductline(String productlineID) {
		/* 1. Now retrieve the new product line, using the ID we created */
		Productline loadedProductline = dao.findById(productlineID);

		/*
		 * 2. Now let's change same value on the product line, and save the
		 * change
		 */
		loadedProductline.setTextdescription("Product line for men's shoes.");

		TransactionStatus status = txManager
				.getTransaction(new DefaultTransactionDefinition());
		dao.update(loadedProductline);
		txManager.commit(status);

		/*
		 * 3. Now let's load the product line from the DB again, and make sure
		 * it text description changed
		 */
		Productline secondLoadedProductline = dao.findById(productlineID);

		System.out.println("*REVISED* Product Line [" + "productLine="
				+ secondLoadedProductline.getProductline()
				+ ", textDescription="
				+ secondLoadedProductline.getTextdescription() + "]");
	}

	private static void deleteProductline(String productlineID) {
		/* 1. Now retrieve the new product line, using the ID we created */
		TransactionStatus status = txManager
				.getTransaction(new DefaultTransactionDefinition());
		Productline loadedProductline = dao.findById(productlineID);

		/* 2. Now let's delete the product line from the DB */
		dao.delete(loadedProductline);
		txManager.commit(status);

		/*
		 * 3. To confirm the deletion, try and load it again and make sure it
		 * fails
		 */
		Productline deletedProductline = dao.findById(productlineID);

		/*
		 * 4. We use a simple inline IF clause to test for null and print
		 * SUCCESSFUL/FAILED
		 */
		System.out.println("Productline deletion: "
				+ (deletedProductline == null ? "SUCCESSFUL" : "FAILED"));
	}
}
